// Copyright © 2020 Attestant Limited.
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package wallet

import (
	eth2client "github.com/attestantio/go-eth2-client"
	"github.com/attestantio/vouch/services/metrics"
	"github.com/attestantio/vouch/services/validatorsmanager"
	"github.com/pkg/errors"
	"github.com/rs/zerolog"
)

type parameters struct {
	logLevel               zerolog.Level
	monitor                metrics.AccountManagerMonitor
	locations              []string
	accountPaths           []string
	passphrases            [][]byte
	validatorsManager      validatorsmanager.Service
	slotsPerEpochProvider  eth2client.SlotsPerEpochProvider
	domainProvider         eth2client.DomainProvider
	farFutureEpochProvider eth2client.FarFutureEpochProvider
}

// Parameter is the interface for service parameters.
type Parameter interface {
	apply(*parameters)
}

type parameterFunc func(*parameters)

func (f parameterFunc) apply(p *parameters) {
	f(p)
}

// WithLogLevel sets the log level for the module.
func WithLogLevel(logLevel zerolog.Level) Parameter {
	return parameterFunc(func(p *parameters) {
		p.logLevel = logLevel
	})
}

// WithMonitor sets the monitor for the module.
func WithMonitor(monitor metrics.AccountManagerMonitor) Parameter {
	return parameterFunc(func(p *parameters) {
		p.monitor = monitor
	})
}

// WithLocations sets the locations to look for wallets.
func WithLocations(locations []string) Parameter {
	return parameterFunc(func(p *parameters) {
		p.locations = locations
	})
}

// WithAccountPaths sets the accounts paths for which to validate.
func WithAccountPaths(accountPaths []string) Parameter {
	return parameterFunc(func(p *parameters) {
		p.accountPaths = accountPaths
	})
}

// WithPassphrases sets the passphrases to unlock accounts.
func WithPassphrases(passphrases [][]byte) Parameter {
	return parameterFunc(func(p *parameters) {
		p.passphrases = passphrases
	})
}

// WithValidatorsManager sets the validator manager.
func WithValidatorsManager(manager validatorsmanager.Service) Parameter {
	return parameterFunc(func(p *parameters) {
		p.validatorsManager = manager
	})
}

// WithSlotsPerEpochProvider sets the slots per epoch provider.
func WithSlotsPerEpochProvider(provider eth2client.SlotsPerEpochProvider) Parameter {
	return parameterFunc(func(p *parameters) {
		p.slotsPerEpochProvider = provider
	})
}

// WithFarFutureEpochProvider sets the far future epoch provider.
func WithFarFutureEpochProvider(provider eth2client.FarFutureEpochProvider) Parameter {
	return parameterFunc(func(p *parameters) {
		p.farFutureEpochProvider = provider
	})
}

// WithDomainProvider sets the domain provider.
func WithDomainProvider(provider eth2client.DomainProvider) Parameter {
	return parameterFunc(func(p *parameters) {
		p.domainProvider = provider
	})
}

// parseAndCheckParameters parses and checks parameters to ensure that mandatory parameters are present and correct.
func parseAndCheckParameters(params ...Parameter) (*parameters, error) {
	parameters := parameters{
		logLevel: zerolog.GlobalLevel(),
	}
	for _, p := range params {
		if params != nil {
			p.apply(&parameters)
		}
	}

	if parameters.monitor == nil {
		return nil, errors.New("no monitor specified")
	}
	if parameters.accountPaths == nil {
		return nil, errors.New("no account paths specified")
	}
	if len(parameters.passphrases) == 0 {
		return nil, errors.New("no passphrases specified")
	}
	if parameters.validatorsManager == nil {
		return nil, errors.New("no validators manager specified")
	}
	if parameters.slotsPerEpochProvider == nil {
		return nil, errors.New("no slots per epoch provider specified")
	}
	if parameters.farFutureEpochProvider == nil {
		return nil, errors.New("no far future epoch provider specified")
	}
	if parameters.domainProvider == nil {
		return nil, errors.New("no domain provider specified")
	}

	return &parameters, nil
}
